@@ -145,6 +145,7 @@ void usage(void) {
 		"    -q,  --quiet            Disable progress message\n"
 		"    -E,  --offset=X         Add offset X (bytes) to OUTPUT\n"
 #endif
+		"    -n,  --note NOTE        Display Message Note (128 words)\n"
 		"    -v,  --version          Display partclone version\n"
 		"    -h,  --help             Display this help\n"
 		, EXECNAME, VERSION, EXECNAME, DEFAULT_BUFFER_SIZE);
@@ -162,20 +163,21 @@ enum {
 
 void parse_options(int argc, char **argv, cmd_opt* opt) {
 #ifdef CHKIMG
-	static const char *sopt = "-hvd::L:s:f:CFiBz:N";
+	static const char *sopt = "-hvd::L:s:f:CFiBz:Nn:";
 #else
 #ifdef RESTORE
-	static const char *sopt = "-hvd::L:o:O:s:f:CFINiqWBz:E:";
+	static const char *sopt = "-hvd::L:o:O:s:f:CFINiqWBz:E:n:";
 #elif DD
-	static const char *sopt = "-hvd::L:o:O:s:f:CFINiqWBz:E:";
+	static const char *sopt = "-hvd::L:o:O:s:f:CFINiqWBz:E:n:";
 #else
-	static const char *sopt = "-hvd::L:cbrDo:O:s:f:RCFINiqWBz:E:";
+	static const char *sopt = "-hvd::L:cbrDo:O:s:f:RCFINiqWBz:E:n:";
 #endif
 #endif
 	static const struct option lopt[] = {
 // common options
 		{ "help",		no_argument,		NULL,   'h' },
 		{ "print_version",	no_argument,		NULL,   'v' },
+		{ "note",		required_argument,	NULL,   'n' },
 		{ "source",		required_argument,	NULL,   's' },
 		{ "debug",		optional_argument,	NULL,   'd' },
 		{ "logfile",		required_argument,	NULL,   'L' },
@@ -256,6 +258,9 @@ void parse_options(int argc, char **argv, cmd_opt* opt) {
 			case 'v':
 				print_version();
 				break;
+			case 'n':
+				strncpy(opt->note, optarg, NOTE_SIZE);
+				break;
 			case 's':
 				opt->source = optarg;
 				break;
@@ -884,11 +889,17 @@ int open_target(char* target, cmd_opt* opt) {
 		    ddd_block_device = 1;
 		else
 		    ddd_block_device = 0;
-	    }else{
+	    } else {
 		ddd_block_device = 0;   
 	    }
+
 	    log_mesg(1, 0, 0, debug, "ddd target file(0) or device(1) ? %i \n", ddd_block_device);
 	}
+
+	if (opt->restore_raw_file == 1) {
+	    ddd_block_device = 0;
+	}
+
 	if (opt->clone || opt->domain || (ddd_block_device == 0)) {
 		if (strcmp(target, "-") == 0) {
 			if ((ret = fileno(stdout)) == -1)
@@ -984,19 +995,19 @@ void sync_data(int fd, cmd_opt* opt) {
 }
 
 void rescue_sector(int *fd, unsigned long long pos, char *buff, cmd_opt *opt) {
-	const char *badsector_magic = "BADSECTOR\0";
+	const char badsector_magic[10] = {'B', 'A', 'D', 'S', 'E', 'C', 'T', 'O', 'R', '\0'};
 
 	if (lseek(*fd, pos, SEEK_SET) == (off_t)-1) {
 		log_mesg(0, 0, 1, opt->debug, "WARNING: lseek error at %llu\n", pos);
 		memset(buff, '?', PART_SECTOR_SIZE);
-		memmove(buff, badsector_magic, sizeof(badsector_magic));
+		memcpy(buff, badsector_magic, strlen(badsector_magic)+1);
 		return;
 	}
 
 	if (io_all(fd, buff, PART_SECTOR_SIZE, 0, opt) == -1) { /// read_all
 		log_mesg(0, 0, 1, opt->debug, "WARNING: Can't read sector at %llu, lost data.\n", pos);
 		memset(buff, '?', PART_SECTOR_SIZE);
-		memmove(buff, badsector_magic, sizeof(badsector_magic));
+		memcpy(buff, badsector_magic, strlen(badsector_magic)+1);
 	}
 }
 
@@ -1068,6 +1079,8 @@ void print_opt(cmd_opt opt) {
 	log_mesg(1, 0, 0, debug, "NCURSES: %i\n", opt.ncurses);
 #endif
 	log_mesg(1, 0, 0, debug, "OFFSET DOMAIN: 0x%llX\n", opt.offset_domain);
+	opt.note[NOTE_SIZE] = '\0';
+	log_mesg(1, 0, 0, debug, "NOTE: %s\n", opt.note);
 }
 
 /// print partclone info
@@ -1095,6 +1108,10 @@ void print_partclone_info(cmd_opt opt) {
 		log_mesg(0, 0, 1, debug, _("Display image information\n"));
 	else
 		log_mesg(0, 0, 1, debug, "Unknown mode\n");
+	if ( strlen(opt.note) > 0 ){
+	    opt.note[NOTE_SIZE] = '\0';
+	    log_mesg(0, 0, 1, debug, "note: %s\n", opt.note);
+	}
 }
 
 /// print image head
